from gravithon.fields.Field import Field
from numpy import array, ndarray


class GravitationalField(Field):
    # Field of N/kg
    def __init__(self, value: ndarray):
        self.value = value

    def __str__(self):
        return super().__str__() \
            + 'N/kg' + '\n'



from gravithon.units.length import meter
from gravithon.units.mass import kilogram
from gravithon.units.time import second

G = gravitational_constant = 6.6743 * 10 ** -11 * (meter ** 3) * (kilogram ** -1) * (second ** -2)
c = speed_of_light = 299792458 * (meter / second)
# TODO: more constants



from gravithon import formulas


class Satellite:
    # Class for both artificial and natural satellites
    # TODO: make interface
    def __init__(self, rotation_period: float, orbital_radius: float, orbital_velocity: float):
        self.rotation_period = rotation_period
        self.orbital_radius = orbital_radius
        self.orbital_velocity = orbital_velocity

    def __str__(self):
        return '\n' + \
            f'  Rotation Period: {self.rotation_period} sec' + '\n' + \
            f'  Orbital Distance: {self.orbital_radius} m' + '\n' + \
            f'  Orbital Velocity: {self.orbital_velocity} m/sec' + '\n' + \
            f'  Orbital Period: {self.orbital_period()} sec'

    def orbital_period(self):
        return formulas.orbital_period(self.orbital_radius, self.orbital_velocity)



from abc import ABC, abstractmethod
from numpy import array, ndarray


class Field(ABC):
    value: ndarray

    # TODO: field boundaries

    def __str__(self):
        return f'  Value: {self.value} '  # units are field dependent

    def dimensions(self):
        return len(self.value)



from gravithon.Sphere import Sphere
from gravithon.astronomy.Satellite import Satellite


class Planet(Sphere, Satellite):
    # radius is volumetric mean radius
    def __init__(self, name: str, mass: float, radius: float,
                 rotation_period: float, distance_from_parent: float, orbital_velocity: float, color: str = None):
        # sphere init
        Sphere.__init__(self, name, mass, radius, color)
        # satellite init
        Satellite.__init__(self, rotation_period, distance_from_parent, orbital_velocity)

    def __str__(self):
        return Sphere.__str__(self) + \
            Satellite.__str__(self) + \
            '\n' + \
            f'  Surface Gravity: {self.surface_gravity()} sec'

    def surface_gravity(self):
        return self.gravitational_field(self.radius)



from gravithon.units.prefixes import *

# MKS units
s = sec = second = 1

min = minute = 60 * second
h = hr = hour = 60 * minute
d = day = 24 * hour
# This is 365 days. For astronomical year use gravithon.Earth.orbital_period()
year_of_days = 365 * day

ms = millisecond = milli * second
μs = microsecond = micro * second
ns = nanosecond = nano * second



from gravithon.units.prefixes import *

# MKS units
m = meter = 1

dm = decimeter = deca * meter
hm = hectometer = hecto * meter
km = kilometer = kilo * meter

cm = centimeter = centi * meter
mm = millimeter = milli * meter
μm = micrometer = micro * meter
nm = nanometer = nano * meter

# US units
inch = 0.0254 * meter

ft = foot = 12 * inch
yd = yard = 36 * inch



from gravithon.units.prefixes import *

# MKS units
kg = kilogram = 1
g = gram = kilogram * (10 ** -3)

t = tonne = 1000 * kg

mg = milligram = milli * gram
μg = microgram = micro * gram
ng = nanogram = nano * gram



from gravithon.astronomy.Planet import Planet
from gravithon.Sphere import Sphere
from gravithon.units.prefixes import *
from gravithon.units.length import meter, kilometer
from gravithon.units.mass import kilogram
from gravithon.units.time import second, hour

# Data from NASA: https://nssdc.gsfc.nasa.gov/planetary/factsheet/
# if you don't believe them you can ask the guys in Russia instead: https://artofproblemsolving.com/community/c1595181

Sun = Sphere(
    name='Sun',
    mass=1988500.0 * yotta * kilogram,
    radius=695700.0 * kilometer,
    color='#ED762E',
)

Mercury = Planet(
    name='Mercury',
    mass=0.33010 * yotta * kilogram,
    radius=2439.7 * kilometer,
    color='#8D8A88',
    rotation_period=1407.6 * hour,
    distance_from_parent=57.9 * mega * kilometer,
    orbital_velocity=47.4 * kilometer / second,
)

Venus = Planet(
    name='Venus',
    mass=4.8673 * yotta * kilogram,
    radius=6051.8 * kilometer,
    color='#F4DBC4',
    rotation_period=-5832.6 * hour,
    distance_from_parent=108.2 * mega * kilometer,
    orbital_velocity=35.0 * kilometer / second,
)

Earth = Planet(
    # https://www.reddit.com/r/ProgrammerHumor/comments/9nk4es/i_think_not/?rdt=48839
    name='Earth',
    mass=5.9722 * yotta * kilogram,
    radius=6371.0 * kilometer,
    color='#0000A0',
    rotation_period=23.9345 * hour,
    distance_from_parent=149.6 * mega * kilometer,
    orbital_velocity=29.8 * kilometer / second,
)

Mars = Planet(
    name='Mars',
    mass=0.64169 * yotta * kilogram,
    radius=3389.5 * kilometer,
    color='#C26D5C',
    rotation_period=24.6229 * hour,
    distance_from_parent=228.0 * mega * kilometer,
    orbital_velocity=24.1 * kilometer / second,
)

Jupiter = Planet(
    name='Jupiter',
    mass=1898.13 * yotta * kilogram,
    radius=69911.0 * kilometer,
    color='#C4B8A0',
    rotation_period=9.9250 * hour,
    distance_from_parent=778.5 * mega * kilometer,
    orbital_velocity=13.1 * kilometer / second,
)

Saturn = Planet(
    name='Saturn',
    mass=568.32 * yotta * kilogram,
    radius=58232.0 * kilometer,
    color='#C0A480',
    rotation_period=10.656 * hour,
    distance_from_parent=1432.0 * mega * kilometer,
    orbital_velocity=9.7 * kilometer / second,
)

Uranus = Planet(
    name='Uranus',
    mass=86.811 * yotta * kilogram,
    radius=25362.0 * kilometer,
    color='#a9d4d6',
    rotation_period=-17.24 * hour,
    distance_from_parent=2867.0 * mega * kilometer,
    orbital_velocity=6.8 * kilometer / second,
)

Neptune = Planet(
    name='Neptune',
    mass=102.409 * yotta * kilogram,
    radius=24622 * kilometer,
    color='#637DAB',
    rotation_period=16.11 * hour,
    distance_from_parent=4515.0 * mega * kilometer,
    orbital_velocity=5.4 * kilometer / second,
)

Pluto = Planet(
    name='Pluto',
    mass=0.01303 * yotta * kilogram,
    radius=1188.0 * kilometer,
    color='#d6c5b6',
    rotation_period=-153.2928 * hour,
    distance_from_parent=5906.4 * mega * kilometer,
    orbital_velocity=4.7 * kilometer / second,
)

Moon = Planet(
    name='Moon',
    mass=0.07346 * yotta * kilogram,
    radius=1737.4 * kilometer,
    color='#797573',
    rotation_period=655.720 * hour,
    distance_from_parent=0.384 * mega * kilometer,
    orbital_velocity=1.022 * kilometer / second,
)
# TODO: ISS?
